Module syntax

Module syntax 

Modules

kw
Custom keywords for parsing. This allows us to use requires, ensures, etc., as if they were built-in Rust keywords during parsing.

Structs

CaptureExpr
The form in a captures clause: as .
SpecArg
A single spec argument.
SpecArgs
Raw spec arguments, i.e. as they appear in the #[spec(...)] proc macro invocation.

Enums

Captures
A group of capture expressions, either a single one or a list. These are not composed of top level syn::Expr expressions.
Keyword
SpecArgValue
Each SpecArg’s value needs to be parsed in a way that allows invalid specs, e.g. forms which do not correspond directly to an syn::Expr in standard Rust.