pub enum SpecArgValue {
Expr(Expr),
Pat(Pat),
Captures(Captures),
}Expand description
Each SpecArg’s value needs to be parsed in a way that allows invalid specs, e.g.
forms which do not correspond directly to an syn::Expr in standard Rust.
NOTE:
a SpecArgValue may hold unrelated syntactic elements such as [’syn::Expr], [syn::Pat`],
and even fragments that would never appear as part of a valid Rust program.
Variants
Implementations
Sourceimpl SpecArgValue
impl SpecArgValue
Sourcepub fn try_into_expr(self) -> Result<Expr>
pub fn try_into_expr(self) -> Result<Expr>
Return the Expr or fail.
Sourcepub fn try_into_pat(self) -> Result<Pat>
pub fn try_into_pat(self) -> Result<Pat>
Return the Pat or fail.
Sourcepub fn try_into_captures(self) -> Result<Captures>
pub fn try_into_captures(self) -> Result<Captures>
Return the Captures or fail.