pub struct Backend {
pub build_check: fn(Option<&Meta>, &TokenStream, &str, &TokenStream) -> TokenStream,
}Fields
build_check: fn(Option<&Meta>, &TokenStream, &str, &TokenStream) -> TokenStreamImplementations
Sourceimpl Backend
impl Backend
Sourcepub fn instrument_trait(
&self,
spec: Spec,
the_trait: ItemTrait,
) -> Result<ItemTrait>
pub fn instrument_trait( &self, spec: Spec, the_trait: ItemTrait, ) -> Result<ItemTrait>
Expand trait items by mangling each method and adding a wrapper default impl.
Mangling a function involves the following:
- Rename the function following the pattern:
fn add->fn __anodized_add. - Make a new function with the original name that has a default impl; the default impl performs runtime validation and calls the mangled function.