pub struct LoopSpec {
pub maintains: Vec<PreCondition>,
pub decreases: Option<LoopVariant>,
/* private fields */
}Expand description
Specifies the intended behavior of a loop: while or for.
Fields
maintains: Vec<PreCondition>Loop invariants: conditions that must hold both before and after the loop’s body runs.
decreases: Option<LoopVariant>Loop variant: an expression that decreases with each run of the loop’s body.
Implementations